package com.cultural.heritage.controller.wx;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.article.OfficialAccountArticleAddRequest;
import com.cultural.heritage.model.dto.article.OfficialAccountArticleQueryRequest;
import com.cultural.heritage.model.dto.article.OfficialAccountArticleUpdateRequest;
import com.cultural.heritage.model.entity.OfficialAccountArticle;
import com.cultural.heritage.model.vo.article.OfficialAccountArticleVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.wx.WeChatOfficialAccountService;
import com.cultural.heritage.utils.ReflexObjectUtil;
import io.swagger.v3.oas.annotations.Hidden;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestController
@RequestMapping("/article")
@Slf4j
@Tag(name = "公众号模块")
public class WeChatController {



    @Resource
    private WeChatOfficialAccountService weChatOfficialAccountService;



    @Resource
    private CommonService commonService;


    /**
     * web端管理员添加文章
     * @param officialAccountArticleAddRequest 文章添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "web端管理员添加文章", description = "参数：地址添加请求体，权限：所有人，方法名：addArticle")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addArticle(@RequestBody OfficialAccountArticleAddRequest officialAccountArticleAddRequest) {
        if (officialAccountArticleAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        OfficialAccountArticle officialAccountArticle = new OfficialAccountArticle();
        BeanUtils.copyProperties(officialAccountArticleAddRequest, officialAccountArticle);
        weChatOfficialAccountService.validArticle(officialAccountArticle, false);
        weChatOfficialAccountService.save(officialAccountArticle);
        return ResultUtils.success(true);
    }



    /**
     * web端管理员删除文章
     * @param deleteRequest   文章id
     * @return  是否成功删除
     */
    @PostMapping("/delete")
    @Operation(summary = "web端管理员删除文章", description = "参数：文章id，权限：所有人，方法名：delArticle")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delArticle(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();
        weChatOfficialAccountService.removeById(id);
        return ResultUtils.success(true);
    }




    /**
     * web端管理员更新文章
     * @param officialAccountArticleUpdateRequest  文章更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "web端管理员更新文章", description = "参数：文章更新请求体，权限：所有人，方法名：updateArticle")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateArticle(@RequestBody OfficialAccountArticleUpdateRequest officialAccountArticleUpdateRequest) {
        if (officialAccountArticleUpdateRequest == null || officialAccountArticleUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        OfficialAccountArticle officialAccountArticle = new OfficialAccountArticle();
        BeanUtils.copyProperties(officialAccountArticleUpdateRequest, officialAccountArticle);
        weChatOfficialAccountService.validArticle(officialAccountArticle, true);
        weChatOfficialAccountService.updateById(officialAccountArticle);
        return ResultUtils.success(true);
    }


    /**
     * Web端管理员分页查看文章
     * @return 公众号文章列表
     */
    @PostMapping("/query/web")
    @Operation(summary = "Web端管理员分页查看文章", description = "参数：无，权限：web端管理员，方法名:listArticleByPage")
    public BaseResponse<Page<OfficialAccountArticleVO>> listArticleByPage(@RequestBody OfficialAccountArticleQueryRequest officialAccountArticleQueryRequest) {
        if (officialAccountArticleQueryRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        long current = officialAccountArticleQueryRequest.getCurrent();
        long pageSize = officialAccountArticleQueryRequest.getPageSize();
        QueryWrapper<OfficialAccountArticle> queryWrapper = weChatOfficialAccountService.getQueryWrapper(officialAccountArticleQueryRequest);
        Page<OfficialAccountArticle> page = weChatOfficialAccountService.page(new Page<>(current, pageSize),queryWrapper);
        List<OfficialAccountArticle> officialAccountArticleList = page.getRecords();
        List<OfficialAccountArticleVO> officialAccountArticleVOS = commonService.convertList(officialAccountArticleList, OfficialAccountArticleVO.class);
        Page<OfficialAccountArticleVO> voPage = new Page<>();
        voPage.setRecords(officialAccountArticleVOS);
        voPage.setPages(page.getPages());
        voPage.setCurrent(page.getCurrent());
        voPage.setTotal(page.getTotal());
        voPage.setSize(page.getSize());
        return ResultUtils.success(voPage);
    }



    /**
     * 小程序用户查看文章
     * @return 公众号文章列表
     */
    @GetMapping("/query")
    @Operation(summary = "小程序用户查看文章", description = "参数：无，权限：所有人，方法名:queryAllArticle")
    public BaseResponse<List<OfficialAccountArticleVO>> queryAllArticle() {
        List<OfficialAccountArticle> officialAccountArticleList = weChatOfficialAccountService.list();
        List<OfficialAccountArticleVO> officialAccountArticleVOS = commonService.convertList(officialAccountArticleList, OfficialAccountArticleVO.class);
        Collections.reverse(officialAccountArticleVOS);
        return ResultUtils.success(officialAccountArticleVOS);
    }





    /**
     * Web端管理员根据id查询文章
     * @param commonRequest 文章id
     * @return 文章
     */
    @PostMapping("/query/id")
    @Operation(summary = "Web端管理员根据id查询文章", description = "参数：文章id，权限：管理员（boss, admin)，方法名:listArticleById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<OfficialAccountArticleVO> listArticleById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        Long id = commonRequest.getId();
        OfficialAccountArticle officialAccountArticle = weChatOfficialAccountService.getById(id);
        OfficialAccountArticleVO officialAccountArticleVO = new OfficialAccountArticleVO();
        BeanUtils.copyProperties(officialAccountArticle, officialAccountArticleVO);
        return ResultUtils.success(officialAccountArticleVO);
    }












    @PostMapping("/getArticle")
    @Hidden
    public BaseResponse<Map<String, Object>> getArticleInfo() {
        String result1 = null;
        try {
            result1 = getToken();
            Map<String,Object> token = JSON.parseObject(result1);
            String result2 = getContentList(token.get("access_token").toString());
            JSONObject jsonObject = JSON.parseObject(result2);
            System.out.println(jsonObject);
            Map<String, Object> keyAndValue = ReflexObjectUtil.getKeyAndValue(jsonObject);
            return ResultUtils.success(keyAndValue);
        } catch (IOException e) {
            log.info("获取公众号文章信息错误");
            throw new BusinessException(ErrorCode.SYSTEM_ERROR);
        }
    }


    private String getToken() throws IOException {
        // access_token接口https请求方式: GET https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET
        String path = " https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential";
        String appid = "wx5d04ca2de0e628a8";
        String secret = "495af5bc4df1b86ffcfc21bb12daea76";
        URL url = new URL(path+"&appid=" + appid + "&secret=" + secret);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");
        connection.connect();

        InputStream in = connection.getInputStream();
        byte[] b = new byte[100];
        int len = -1;
        StringBuffer sb = new StringBuffer();
        while((len = in.read(b)) != -1) {
            sb.append(new String(b,0,len));
        }

        System.out.println(sb);
        in.close();
        return sb.toString();
    }



    private String getContentList(String token) throws IOException {
        String path = " https://api.weixin.qq.com/cgi-bin/freepublish/batchget?access_token=" + token;
        URL url = new URL(path);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("POST");
        connection.setDoOutput(true);
        connection.setRequestProperty("content-type", "application/json;charset=utf-8");
        connection.connect();
        // post发送的参数
        Map<String, Object> map = new HashMap<>();
        map.put("offset", 0); // news表示图文类型的素材，具体看API文档
        map.put("count", 20);
        map.put("no_content", 1);
        // 将map转换成json字符串
        String paramBody = JSON.toJSONString(map); // 这里用了Alibaba的fastjson

        OutputStream out = connection.getOutputStream();
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(out));
        bw.write(paramBody); // 向流中写入参数字符串
        bw.flush();

        InputStream in = connection.getInputStream();
        byte[] b = new byte[100];
        int len = -1;
        StringBuffer sb = new StringBuffer();
        while((len = in.read(b)) != -1) {
            sb.append(new String(b,0,len));
        }

        in.close();
        return sb.toString();
    }








}

