package com.cultural.heritage.service.good.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.mapper.CartRecordMapper;
import com.cultural.heritage.model.dto.cart.CartOrderItemAddRequest;
import com.cultural.heritage.model.dto.cart.CartRecordAddRequest;
import com.cultural.heritage.model.entity.CartRecord;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.cart.CartGoodVO;
import com.cultural.heritage.model.vo.cart.CartRecordVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class CartRecordServiceImpl extends ServiceImpl<CartRecordMapper, CartRecord> implements CartRecordService {



    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;


    @Resource
    private CommonService commonService;

    /**
     * 更新购物车的商品属性
     */
    @Override
    public void updateCartGoodAttribute(CartRecordAddRequest cartRecordAddRequest, HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        Long goodId = cartRecordAddRequest.getGoodId();
        Good good = goodService.getById(goodId);
        goodService.checkGoodIsExist(good);
        QueryWrapper<CartRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        queryWrapper.eq("goodId", goodId);
        CartRecord cartRecord = this.baseMapper.selectOne(queryWrapper);
        if (cartRecord != null) {
            // 判断数量是否超出库存
            int currentQuantity = cartRecord.getQuantity() + cartRecordAddRequest.getQuantity();
            goodService.checkInventoryIsEnough(currentQuantity, good.getInventory());
            // 如果购物车中已存在该商品, 就叠加数量
            cartRecord.setQuantity(currentQuantity);
            cartRecord.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(currentQuantity)));
            this.updateById(cartRecord);
        } else {
            // 判断数量是否超出库存
            Integer currentQuantity = cartRecordAddRequest.getQuantity();
            goodService.checkInventoryIsEnough(currentQuantity, good.getInventory());
            // 如果购物车不存在该商品，就向购物车中添加一项
            CartRecord cartGood = new CartRecord();
            BeanUtils.copyProperties(cartRecordAddRequest, cartGood);
            cartGood.setUserId(userId);
            cartGood.setSubtotal(good.getPrice().multiply(BigDecimal.valueOf(cartGood.getQuantity())));
            // 校验
            this.validCart(cartGood, false);
            this.save(cartGood);
        }
    }





    /**
     * 封装用户购物车中的商品信息
     */
    @Override
    public List<CartRecordVO> transformToCartRecordVOList(List<CartRecord> list) {

        // 获取所有商品列表
        List<Good> goodList = commonService.findByFieldInTargetField(list, goodService, CartRecord::getGoodId, "id");
        List<Long> goodIds = goodList.stream().map(Good::getId).toList();
        // 移除购物车中不存在的商品项
        List<Long> errorCartIds = new ArrayList<>();
        for (CartRecord cartRecord : list) {
            Long cartRecordId = cartRecord.getId();
            Long goodId = cartRecord.getGoodId();
            if (!goodIds.contains(goodId)) errorCartIds.add(cartRecordId);
        }
        // 删除购物车中的商品项
        this.removeBatchByIds(errorCartIds);


        // 封装map集合（键：商品id, 值：商品详情信息）
        Map<Long, Good> map = new HashMap<>();
        for (Good good : goodList) {
            map.put(good.getId(), good);
        }

        return list.stream().map(cartRecord -> {
            // 封装购物车记录VO
            CartRecordVO cartRecordVO = new CartRecordVO();
            BeanUtils.copyProperties(cartRecord, cartRecordVO);

            // 封装商品类GoodVO
            Long goodId = cartRecord.getGoodId();
            Good good = map.get(goodId);
            CartGoodVO goodVO = new CartGoodVO();
            BeanUtils.copyProperties(good, goodVO);
            cartRecordVO.setCartGoodVO(goodVO);
            return cartRecordVO;
        }).toList();
    }



    /**
     * 校验购物车中的商品信息
     */
    @Override
    public void validCart(CartRecord cartRecord, boolean update) {
        Long userId = cartRecord.getUserId();
        Long goodId = cartRecord.getGoodId();
        Integer quantity = cartRecord.getQuantity();
        BigDecimal subtotal = cartRecord.getSubtotal();
        if (update) {
            if (userId == null) {
                throw new BusinessException(ErrorCode.PARAMS_ERROR);
            }
        }
        Good good = goodService.getById(goodId);
        if (good == null || good.getIsShelves() == 0) {
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "不存在该商品或该商品已下架");
        }
        if (ObjectUtils.isEmpty(quantity) || quantity < 1) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请传递正确的商品数量");
        }
        if (ObjectUtils.isEmpty(subtotal) || subtotal.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
    }


    /**
     * 校验购物车商品项的准确性
     */
    private boolean validIsConsistent(CartRecord cartRecord, Good good) {
        // 校验购物车中的商品是否存在
        if (good == null || good.getIsShelves() == 0) {
            return false;
        }
        // 校验购物车中的商品数量
        Integer inventory = good.getInventory();
        Integer quantity = cartRecord.getQuantity();
        return quantity <= inventory;
    }


    /**
     * 在提交订单页校验购物车商品项的准确性
     */
    @Override
    public boolean validIsConsistent(List<CartOrderItemAddRequest> cartOrderItemAddRequestList) {
        if (cartOrderItemAddRequestList.size() == 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "购物车中没有商品");
        }
        // 封装map集合（键：购物车id, 值：购买数量）
        Map<Long, Integer> map = new HashMap<>();
        for (CartOrderItemAddRequest cartOrderItemAddRequest : cartOrderItemAddRequestList) {
            map.put(cartOrderItemAddRequest.getCartRecordId(), cartOrderItemAddRequest.getQuantity());
        }
        // 修改购物车商品项的购买数量
        List<CartRecord> cartRecordList = commonService.findByFieldInTargetField(cartOrderItemAddRequestList, this, CartOrderItemAddRequest::getCartRecordId, "id");
        for (CartRecord cartRecord : cartRecordList) {
            Integer quantity = map.get(cartRecord.getId());
            cartRecord.setQuantity(quantity);
        }
        List<Long> cartIds = getInvalidCartIds(cartRecordList);
        return cartIds.isEmpty();
    }


    /**
     * 找出存在问题的购物车id列表
     */
    @Override
    public List<Long> getInvalidCartIds(List<CartRecord> cartRecords) {
        // 获取购物车的商品列表
        List<Good> goodList = commonService.findByFieldInTargetField(cartRecords, goodService, CartRecord::getGoodId, "id");
        // 封装map集合（键：商品id, 值：商品详情信息）
        Map<Long, Good> map = new HashMap<>();
        for (Good good : goodList) {
            map.put(good.getId(), good);
        }
        List<CartRecord> cartRecordList = cartRecords.stream().filter(cartRecord -> {
            Long goodId = cartRecord.getGoodId();
            Good good = map.get(goodId);
            return !validIsConsistent(cartRecord, good);
        }).toList();
        return cartRecordList.stream().map(CartRecord::getId).toList();
    }
}
